import math
from datetime import datetime
import MetaTrader5 as mt5

# выведем данные о пакете MetaTrader5
print("MetaTrader5 package author: ", mt5.__author__)
print("MetaTrader5 package version: ", mt5.__version__)

# импортируем модуль pandas для вывода полученных данных в табличной форме
import pandas as pd

pd.set_option('display.max_columns', 50)  # сколько столбцов показываем
pd.set_option('display.width', 1500)  # макс. ширина таблицы для показа
# импортируем модуль pytz для работы с таймзоной
import pytz

# установим подключение к терминалу MetaTrader 5
if not mt5.initialize():
    print("initialize() failed")
    mt5.shutdown()

# установим таймзону в UTC
timezone = pytz.timezone("Etc/UTC")
# создадим объекты datetime в таймзоне UTC, чтобы не применялось смещение локальной таймзоны
utc_from = datetime(2020, 1, 1, tzinfo=timezone)
utc_to = datetime(2020, 12, 31, hour=23, minute=59, second=59, tzinfo=timezone)
# получим бары с EURUSD H1 в интервале 2020.01.01 00:00 - 2020.31.12 13:00 в таймзоне UTC
rates_H1 = mt5.copy_rates_range("EURUSD", mt5.TIMEFRAME_H1, utc_from, utc_to)
# получим также бары с D1 в интервале 2020.01.01 00:00 - 2020.31.12 13:00 в таймзоне UTC
rates_D1 = mt5.copy_rates_range("EURUSD", mt5.TIMEFRAME_D1, utc_from, utc_to)
# завершим подключение к терминалу MetaTrader 5 и продолжим обработку полученных баров
mt5.shutdown()

# создадим из полученных данных DataFrame
rates_frame = pd.DataFrame(rates_H1)

# добавим колонку "Доходность"
rates_frame['return'] = 0.0
# теперь вычислим доходности как return[i] = (close[i] - close[i-1])/close[i-1]
prev_close = 0.0
for i, row in rates_frame.iterrows():
    close = row['close']
    rates_frame.at[i, 'return'] = close / prev_close - 1 if prev_close != 0.0 else 0.0
    prev_close = close

print("\nВычислим среднее и среднеквадратичное отклонение доходностей на H1 барах")
print('H1 rates:', rates_frame.shape[0])
ret_average = rates_frame[1:]['return'].mean()  # не берем первую строку с нулевой доходностью
print('H1 return average=', ret_average)
ret_std = rates_frame[1:]['return'].std(ddof=0) # не берем первую строку с нулевой доходностью
print('H1 return std =', ret_std)
sharpe_H1 = ret_average / ret_std
print('H1 Sharpe = Average/STD = ', sharpe_H1)

sharpe_annual_H1 = sharpe_H1 * math.sqrt(rates_H1.shape[0]-1)
print('Sharpe_annual(H1) =', sharpe_annual_H1)

# теперь вычислим коэффициент Шарпа на таймфрейме D1
# создадим из полученных данных DataFrame
rates_daily = pd.DataFrame(rates_D1)

# добавим колонку "Доходность"
rates_daily['return'] = 0.0
# теперь вычислим доходности
prev_return = 0.0
for i, row in rates_daily.iterrows():
    close = row['close']
    rates_daily.at[i, 'return'] = close / prev_return - 1 if prev_return != 0.0 else 0.0
    prev_return = close

print("\nВычислим среднее и среднеквадратичное отклонение доходностей на D1 барах")
print('D1 rates:', rates_daily.shape[0])
daily_average = rates_daily[1:]['return'].mean()
print('D1 return average=', daily_average)
daily_std = rates_daily[1:]['return'].std(ddof=0)
print('D1 return std =', daily_std)
sharpe_daily = daily_average / daily_std
print('D1 Sharpe =', sharpe_daily)

sharpe_annual_D1 = sharpe_daily * math.sqrt(rates_daily.shape[0]-1)
print('Sharpe_annual(D1) =', sharpe_annual_D1)
